/*
* fichier:        geigerutil.c
* objet:          utilitaires pour compteur geiger
* Auteur:         Pascal Chour
* Cr:           2015-12-02
* Copyright:      (C) Pascal Chour
* version:        1
* date:           04/2016
*/
#include <msp430f449.h>
#include <inttypes.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "common.h"
#include "geigerutil.h"
#include "geigerutilasm.h"

const char P_CM[] = "CM=";


/* 
Cette fonction fait un dlai d'attente LOGICIEL  de x ms.
Elle fonctionne pur une frquence donne du processeur qui 
doit tre fixe  la compilation
*/

void delayMs(unsigned int delay)
{
    while (delay--)
    {
        __delay_cycles(FREQ_PROC_DIV_1000);
    }
}




/*
* Gestion des boutons poussoirs par polling
* Indique quels sont les poussoirs nouvellement appuys
* En entre : tat prcdent des poussoirs presss dans buttonPressed
* En sortie : tat courant dans buttonPressed et rsultat de la fonction
* FALSE = pas de changement d'tat. TRUE changement d'tat, bouton appuy
* Auteur: Pascal Chour
*/
bool_t buttonHandler (unsigned int *buttonPressed)
{
    if (*buttonPressed == ((~P3IN) & BTN_MASK)) {
      return(FALSE);
    } else {
      delayMs(50); // anti rebond. Dure dpend qualit des boutons. 100ms pour pas d'ennuis
      *buttonPressed = ((~P3IN) & BTN_MASK);
      return(*buttonPressed != 0);
    }
}

/*
Cet utilitaire donne le niveau lu dans le DAC qui mesure la tension
d'alimentation. La valeur en mv se calcule ainsi

  tension = mult = (ADC12MEM0 * 5000l) / 4096l;

!!! Cette procdure fait des attentes logicielles et du polling !!!

Elle ne devrait pas tre appele trop souvent car elle consomme quelques
centaines de A du fait de l'utilisation du DAC.

Elle rend un rsultat utilisable pour afficher la jauge de la batterie sur l'afficheur 
0 : batterie vide (ou quasiment !)
1 : batterie faible
2 : batterie moyenne
3 : batterie charge

Inspir de http://fixituntilitsbroken.blogspot.fr/2011/08/reading-supply-voltage-using-msp430s.html
- If Vcc = 3.3V, ADC output should be (1.65/2.5)*4095 = 2703
- Therefore (halfVcc/2.5)*4095 = ADC reading and (Vcc/2.5)*4095 = 2*ADC
- So Vcc*4096 = 5*ADC and VCC=5*ADC/4095
*/
unsigned int niveau_batterie(unsigned int *voltage)
{
 
   unsigned long mult;
  // REFON  => gnrateur de rfrence actif
  // REF2_5V : utilise le gnrateur 2,5V
  // ADC12ON => dmarre le convertisseur
  // SHT0_15 => chantillonage 1024
  
  ADC12CTL0 = REFON + REF2_5V + ADC12ON + SHT0_15;  // turn on 2.5V ref, set samp time=1024 cycles
  // SHP => source du signal d'chantillonage interne
  ADC12CTL1 = SHP;                                  // Use sampling timer, internal ADC12OSC
  // SREF_1 => V R+ = Ve REF+ and VR- =AV SS
  // INCH_11 => (AVCC  AVSS ) / 2
  ADC12MCTL0 = SREF_1 + INCH_11;                   // Channel A10, Vcc/2
  delayMs(ADC_VREF_DELAY_MS);     // 17mSec delay required to Vref capacitors
  ADC12CTL0 |= ENC;               // Enable conversions
  ADC12CTL0 |= ADC12SC;           // Start conversions
  while (!(ADC12IFG & 0x01));     // Conversion done?

  mult = ADC12MEM0 * 5000l;
  mult = mult / 4096l;
  *voltage = mult;
  if (mult > 3200) {
    mult = 3;
  } else { 
    if (mult > 3100) {
      mult = 2;
    } else {
      if (mult > 3000) {
        mult = 1;
      } else {mult = 0;}
    }
  }
  ADC12CTL0 &= ~ENC;
  ADC12CTL0 &= ~ADC12ON;
  ADC12CTL0 = 0;
  return ((unsigned int)(mult));
}


/*
Rception de caractres avec contrle longueur, filtrage LF et mise  jour
de la structure de rception. Si l'application ne rinitialise pas cette
structure, la rception ne marchera plus !
*/

#pragma vector=UART0RX_VECTOR
__interrupt void UART0_receive_interrupt(void)
{
  char c;
  if (!(URXIFG0 & IFG1)) {
    c = toupper(U0RXBUF);
    if (!recept.a_traiter) { // si pas traitement applicatif en cours
      if (recept.index >= BUF_MAX) { // dpassement de buffer. On efface tout !
        recept.index = 0;
        recept.buf[0]= 0x00;
      } else {
        if (c == 0x0D) { //RC = caractre d'arrt
          recept.buf[recept.index] = 0x00;
          recept.a_traiter = TRUE; // le buffer est prt  tre exploit. 
        } else {
          if (c != 0x0A) { // on filtre les LF
            recept.buf[recept.index] = c; // on stocke le caractre reu.
            recept.index++; // et on incrment l'index du buffer.
          }
        }
      }
    }
  }
}



void lecteur_sauterblancs(void)
{
  while ((recept.buf[recept.index] == ' ') && (recept.index < BUF_MAX) && (recept.buf[recept.index] != 0x00)) {
    recept.index++;
  }
}

char lecteur_carcourantetinc(void)
{
  char c;
  c = recept.buf[recept.index];
  if ((recept.index < BUF_MAX) && (recept.buf[recept.index] != 0x00)) {recept.index++;}
  lecteur_sauterblancs();
  return(c);
}  

/*
initialise l'analyseur lexical
*/

void lecteur_init(void)
{
  recept.index = 0;
  lecteur_sauterblancs();
}

/* 
lit un caractre de l'analyseur lexical en filtrant les espaces et 
en vitant les dpassements
*/

/*
Renvoie vrai si on trouve s dans recept.buf  la position courante de recept.index
*/

bool_t lecteur_ident(char *s)
{
  int j;
  int l;
  lecteur_sauterblancs();
  j = 0;
  l = strlen(s);
  while ((recept.buf[recept.index+j] >= 'A') && (recept.buf[recept.index+j] <= 'Z')) {j++;}
  /* en sortie, j = longueur d'un "ident" dans le buffer */
  if (j == l) {
    j = 0;
    while ((recept.buf[recept.index+j] == s[j]) && (j < l)) {
      j++;
    }
    if (j == l) {
      recept.index = recept.index+j;
    }
  }
  return (j == l);  
}

/*
lit un entier limit  9999 et rend le rsultat dans nb.
La fonction rend vrai si un entier a t lu. 
Si un entier n' pas t lu, la fonction rend faux et 
nb est inchang.
*/

bool_t lecteur_int(unsigned int *nb)
{
  bool_t nb_valide;
  unsigned int nb_save;
  lecteur_sauterblancs();
  *nb = 0;
  nb_valide = FALSE;
  nb_save = *nb;
  while ((recept.buf[recept.index] >= '0') && (recept.buf[recept.index] <= '9') && (*nb <= 1000)) {
    *nb = *nb*10+(recept.buf[recept.index] & 0x0f);
    recept.index++;
    nb_valide = TRUE;
  }
  if (!nb_valide) {*nb = nb_save;}
  return(nb_valide);
}

/*
Lit un rel et rend le rsultat dans nb.
La fonction est ddie au facteur de conversion
Elle rend faux si le facteur est > 1 et =0.
Dans ce cas, nb est inchang
Elle rend vrai sinon. nb contient le rsultat
*/

bool_t lecteur_FC(float *nb)
{
  float ph;
  float divise;
  lecteur_sauterblancs();
  ph = 0; // lecture partie entire
  while ((recept.buf[recept.index] >= '0') && (recept.buf[recept.index] <= '1') && (ph < 2.0)) {
    ph = ph*10+(recept.buf[recept.index] & 0x0f);
    recept.index++;
  }
  if (ph < 2.0) {
    if (lecteur_carcourantetinc() == '.') { // a-t-on un point ?
      divise = 10.0; // lecture partie dcimale.
      while ((recept.buf[recept.index] >= '0') && (recept.buf[recept.index] <= '9')) {
        ph = ph+(float)(recept.buf[recept.index] & 0x0f)/divise;
        recept.index++;
        divise = divise*10.0;
      }
    }
  } else { ph = 0.0;}
  if (ph > 0.0) {*nb = ph;}
  return(ph > 0.0);
}

/*
Rinitialise le buffer de rception sur UART0
*/
void UART0_initbuffer(void)
{
    /* etat IO */
    recept.a_traiter = FALSE;      // rien  traiter
    recept.index = 0;              // rien reu 
    recept.buf[0] = 0x00;          // par prcaution
}    

/*
transmet le contenu de s termin par 0x00 sur l'UART0
Pas de contrle de s ni de i
termine la transmission par LF/RC
*/

void UART0_writeln(char *s)
{
  int i;
  i = 0;
  while (s[i] != 0x00) {
    UART0_putChar(s[i]);
    i++;
  }
  UART0_putChar(0x0A);
  UART0_putChar(0x0D);
}

/*
Emet la commande CM= avec la valeur de comptage passe en paramtre sur
l'UART0. L'UART doit avoir t pralablement initialise !
transmet 9999 si la valeur de comptage est suprieure  10000
*/
void EMET_CM(unsigned long v)
{
  char s[BUF_MAX];
  if (v > 999999) {v = 999999;}
  sprintf(s,"CM=%lu",v);
  UART0_writeln(s);
}  


/*
 Traite la rception des commandes du host et renvoie les rponses*
 Le buffer est suppos avoir un caractre nul en fin.
*/
void traite_commande(void)
{
  char c;
  char s[BUF_MAX];
  lecteur_init();
  c = lecteur_carcourantetinc();
  switch (c)
  {
  case 'V': // version logiciel, modle compteur et n de srie
    
    sprintf(s,"VP=%s,%s,%s",version_pgm,counter_model,serial_number);
    UART0_writeln(s);
    break;
    
case 'G': // Get Internal Data
    
    sprintf(s,"ID=%i",voltage);
    UART0_writeln(s);
    break;    
    
  case 'P': // paramtrage compteur
    if (lecteur_carcourantetinc() == '=') {
      if (lecteur_ident("ON")) {
        alarme_led = TRUE;
      } else {
        if (lecteur_ident("OFF")) {
          alarme_led = FALSE;
        }
      }
      if (lecteur_carcourantetinc() == ',') {
        if (lecteur_ident("ON")) {
          alarme_buz = TRUE;
        } else {
          if (lecteur_ident("OFF")) {
            alarme_buz = FALSE;
          }
        }
      }
      if (lecteur_carcourantetinc() == ',') {
        if (lecteur_ident("SV")) {
          affiche_sievert = TRUE;
        } else {
          if (lecteur_ident("PM")) {
            affiche_sievert = FALSE;
          }
        }
      }
      if (lecteur_carcourantetinc() == ',') {
        lecteur_int(&seuil_alarme);
      }
    }
    strcpy(s,"PC=");
    if (alarme_led) {
      strcat(s,"ON,");
    } else {
      strcat(s,"OFF,");
    }
    if (alarme_buz) {
      strcat(s,"ON,");
    } else {
      strcat(s,"OFF,");
    }
    if (affiche_sievert) {
      strcat(s,"SV,");
    } else {
      strcat(s,"PM,");
    }
    sprintf(s,"%s%d",s,seuil_alarme);
    UART0_writeln(s);
    break;
              
  case 'F':
    if (lecteur_carcourantetinc() == '=') {
      lecteur_FC(&conversion_factor);
    }
    sprintf(s,"CF=%f",conversion_factor);
    UART0_writeln(s);
    break;
  }
  UART0_initbuffer(); // on autorise de nouveau la rception
}


